﻿using EonaCat.SecretVault.Data;
using EonaCat.SecretVault.Helpers;
using EonaCat.SecretVault.Middleware;
using EonaCat.SecretVault.Models;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Linq;
using System.Security.Cryptography;
using System.Text;

// This file is part of the EonaCat project(s) which is released under the Apache License.
// See the LICENSE file or go to https://EonaCat.com/License for full license details.

namespace EonaCat.SecretVault.Controllers
{
    [ApiController]
    [Route("api/[controller]")]
    public class AdminController : ControllerBase
    {
        private readonly SecretDbContext _db;
        public AdminController(SecretDbContext db) => _db = db;

        [HttpPost("generate-key")]
        [RequireRole(ApiRole.Admin)]
        public IActionResult GenerateKey([FromQuery] ApiRole role)
        {
            var tenantId = HttpContext.Items["TenantId"]?.ToString()!;
            var newKey = Guid.NewGuid().ToString("N");

            var rootKey = Environment.GetEnvironmentVariable("ROOT_PROTECTOR_KEY")!;
            var tenantKey = Convert.ToBase64String(RandomNumberGenerator.GetBytes(32));

            var encryptedTenantKey = TenantKeyProtector.EncryptTenantKey(tenantKey, rootKey);
            var apiKey = GenerateApiKey();
            var apiKeyHash = HashApiKey(apiKey);

            _db.ApiKeys.Add(new TenantApiKey
            {
                ApiKey = newKey,
                TenantId = tenantId,
                EncryptionKey = tenantKey,
                EncryptedEncryptionKey = encryptedTenantKey,
                CreatedAt = DateTime.UtcNow,
                Scope = "read,write",
                ApiKeyHash = apiKeyHash,
                Role = role,
                IsActive = true,
            });
            _db.SaveChanges();

            return Ok(new { apiKey = newKey, role });
        }

        [HttpDelete("revoke-key")]
        [RequireRole(ApiRole.Admin)]
        public IActionResult RevokeKey([FromQuery] string key)
        {
            var tenantId = HttpContext.Items["TenantId"]?.ToString()!;
            var entry = _db.ApiKeys.FirstOrDefault(k => k.ApiKey == key && k.TenantId == tenantId);
            if (entry == null)
            {
                return NotFound("Key not found");
            }

            entry.IsActive = false;
            _db.SaveChanges();

            return Ok("Key revoked");
        }

        [HttpGet("keys")]
        [RequireRole(ApiRole.Admin)]
        public IActionResult ListKeys()
        {
            var tenantId = HttpContext.Items["TenantId"]?.ToString()!;
            var keys = _db.ApiKeys
                .Where(k => k.TenantId == tenantId)
                .Select(k => new { k.ApiKey, k.Role, k.IsActive, k.CreatedAt })
                .ToList();

            return Ok(keys);
        }

        string GenerateApiKey()
        {
            return Convert.ToBase64String(RandomNumberGenerator.GetBytes(32));
        }

        string HashApiKey(string apiKey)
        {
            using var sha256 = SHA256.Create();
            var hash = sha256.ComputeHash(Encoding.UTF8.GetBytes(apiKey));
            return Convert.ToBase64String(hash);
        }
    }
}
