﻿using EonaCat.SecretVault.Data;
using EonaCat.SecretVault.Helpers;
using EonaCat.SecretVault.Models;
using Microsoft.Extensions.Configuration;
using System;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace EonaCat.SecretVault.Services
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.

    public class SecretService
    {
        private readonly SecretDbContext _db;
        private readonly string _rootKeyBase64;

        public SecretService(IConfiguration config, SecretDbContext db)
        {
            _db = db;
            _rootKeyBase64 = Environment.GetEnvironmentVariable("ROOT_PROTECTOR_KEY")
                             ?? throw new Exception("ROOT_PROTECTOR_KEY env var is missing");
        }

        private byte[] GetTenantKey(string tenantId)
        {
            var tenant = _db.ApiKeys.FirstOrDefault(t => t.TenantId == tenantId);
            if (tenant == null)
            {
                throw new Exception("Tenant API key not found");
            }

            var decryptedBase64 = TenantKeyProtector.DecryptTenantKey(tenant.EncryptedEncryptionKey, _rootKeyBase64);
            return Convert.FromBase64String(decryptedBase64);
        }

        public async Task StoreAsync(string tenantId, string key, string secret, string user)
        {
            var tenantKey = GetTenantKey(tenantId);

            using var aes = Aes.Create();
            aes.Key = tenantKey;
            aes.GenerateIV();

            var encryptor = aes.CreateEncryptor();
            var plain = Encoding.UTF8.GetBytes(secret);
            var cipher = encryptor.TransformFinalBlock(plain, 0, plain.Length);
            var payload = Convert.ToBase64String(aes.IV) + "." + Convert.ToBase64String(cipher);

            var model = new SecretModel { TenantId = tenantId, Key = key, EncryptedValue = payload, CreatedAt = DateTime.UtcNow };
            _db.Secrets.Add(model);

            _db.AuditLogs.Add(new AuditLog
            {
                TenantId = tenantId,
                Action = "Store",
                Key = key,
                PerformedBy = user,
                Timestamp = DateTime.UtcNow
            });

            await _db.SaveChangesAsync();
        }

        public string? Retrieve(string tenantId, string key, string user)
        {
            var tenantKey = GetTenantKey(tenantId);

            var model = _db.Secrets
                .Where(s => s.TenantId == tenantId && s.Key == key)
                .OrderByDescending(s => s.CreatedAt)
                .FirstOrDefault();
            if (model == null)
            {
                return null;
            }

            var parts = model.EncryptedValue.Split('.');
            var iv = Convert.FromBase64String(parts[0]);
            var cipher = Convert.FromBase64String(parts[1]);

            using var aes = Aes.Create();
            aes.Key = tenantKey;
            aes.IV = iv;
            var decryptor = aes.CreateDecryptor();
            var decrypted = decryptor.TransformFinalBlock(cipher, 0, cipher.Length);

            _db.AuditLogs.Add(new AuditLog
            {
                TenantId = tenantId,
                Action = "Retrieve",
                Key = key,
                PerformedBy = user,
                Timestamp = DateTime.UtcNow
            });
            _db.SaveChanges();

            return Encoding.UTF8.GetString(decrypted);
        }

        internal async Task<bool> DeleteAsync(string tenantId, string key, string user)
        {
            var model = _db.Secrets
                .Where(s => s.TenantId == tenantId && s.Key == key)
                .OrderByDescending(x => x.CreatedAt)
                .FirstOrDefault();

            if (model == null)
            {
                return false;
            }

            _db.Secrets.Remove(model);
            _db.AuditLogs.Add(new AuditLog
            {
                TenantId = tenantId,
                Action = "Delete",
                Key = key,
                PerformedBy = user,
                Timestamp = DateTime.UtcNow
            });
            await _db.SaveChangesAsync();
            return true;
        }
    }
}
